import java.awt.event.*;
import java.util.*;

/**
 * Lab #5
 * CS 2334, Section SECTION NUMBER GOES HERE
 * DATE GOES HERE
 * <p>
 * Each object of this class represents a GUI enhanced model of a state.
 * The model of an MVC GUI.
 * </p>
 * @version 1.0
 *
 */
public class StateModel extends State {

	/**
	 * List to keep track of who is registered to listen for events from the StateModel.
	 */
	private ArrayList<ActionListener> actionListenerList;
	
	/**
	 * @param name
	 */
	public StateModel(String name) {
		super(name);
	}

	/**
	 * @param name
	 * @param cities
	 */
	public StateModel(String name, LinkedHashMap<String, City> cities) {
		super(name, cities);
	}
	
	/**
	 * @param city
	 */
	public void addCity(City city) {
		super.addCity(city);
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "add city"));
	}
	
	
	public void clearCities() {
		super.clearCities();
		processEvent(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "clear city list"));
	}
	
	/**
	 * Register an action event listener.
	 */
	public synchronized void addActionListener(ActionListener l) {
		if (actionListenerList == null)
			actionListenerList = new ArrayList<ActionListener>();
		actionListenerList.add(l);
	}
	
	/**
	 * Remove an action event listener.
	 */
	public synchronized void removeActionListener(ActionListener l) {
		if (actionListenerList != null && actionListenerList.contains(l))
			actionListenerList.remove(l);
	}
	
	/**
	 * Fire event.
	 */
	private void processEvent(ActionEvent e) {
		ArrayList<ActionListener> list;
		
		synchronized (this) {
			if (actionListenerList == null) return;
			list = (ArrayList<ActionListener>)actionListenerList.clone();
		}
		
		for (int i = 0; i < list.size(); i++) {
			ActionListener listener = list.get(i);
			listener.actionPerformed(e);
		}
	}
}